@@ -144,6 +144,9 @@ toggle_button_toggled_cb(GtkToggleButton *button, gpointer data)
     gtk_selection_owner_set (curr_data->event_box,
 	  		     GDK_SELECTION_PRIMARY,
                              GDK_CURRENT_TIME); 
+    gtk_selection_owner_set (curr_data->event_box,
+	  		     GDK_SELECTION_CLIPBOARD,
+                             GDK_CURRENT_TIME); 
     curr_data->last_index = button_index;
   }	
 	     
@@ -171,6 +174,7 @@ key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
   charpick_data *p_curr_data = data;
   gchar inputchar = event->keyval;
+
   switch (inputchar)
     {
     case 'a' : p_curr_data->charlist = a_list;
@@ -252,6 +256,8 @@ key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
     case ' ' : p_curr_data->charlist = 
 	         p_curr_data->default_charlist;
                break;
+    default :
+    		return FALSE;
     }
 
   p_curr_data->last_index = NO_LAST_INDEX;
@@ -287,7 +293,7 @@ build_table(charpick_data *p_curr_data)
     currstr[0] = p_curr_data->charlist[i];
     currstr[1] = '\0';
  
-    str_utf8 = g_locale_to_utf8 (currstr, -1, NULL, NULL, NULL);
+    str_utf8 = g_convert (currstr, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
     toggle_button = gtk_toggle_button_new_with_label (str_utf8);
     g_free (str_utf8);
     gtk_box_pack_start (GTK_BOX (box), toggle_button, TRUE, TRUE, 0);
@@ -382,6 +388,28 @@ help_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 #endif
 }
 
+static void
+applet_destroy (GtkWidget *widget, gpointer data)
+{
+  charpick_data *curr_data = data;
+  GtkTooltips *tooltips;
+  GtkWidget *applet = curr_data->applet;
+
+  g_return_if_fail (curr_data);
+  
+  if (curr_data->default_charlist)
+      g_free (curr_data->default_charlist);
+  curr_data->default_charlist = NULL;
+  
+  tooltips = g_object_get_data (G_OBJECT (applet), "tooltips");
+  if (tooltips) {
+      g_object_unref (tooltips);
+      g_object_set_data (G_OBJECT (applet), "tooltips", NULL);
+  }
+  
+  g_free (curr_data);
+  
+}
 
 static const BonoboUIVerb charpick_applet_menu_verbs [] = {
         BONOBO_UI_VERB ("Props", property_show),
@@ -391,22 +419,38 @@ static const BonoboUIVerb charpick_applet_menu_verbs [] = {
         BONOBO_UI_VERB_END
 };
 
-static const char charpick_applet_menu_xml [] =
-	"<popup name=\"button3\">\n"
-	"   <menuitem name=\"Item 1\" verb=\"Props\" _label=\"Properties\"\n"
-	"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
-	"   <menuitem name=\"Item 2\" verb=\"Help\" _label=\"Help\"\n"
-	"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
-	"   <menuitem name=\"Item 3\" verb=\"About\" _label=\"About\"\n"
-	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
-	"</popup>\n";
+void
+set_atk_name_description (GtkWidget *widget, const gchar *name,
+                          const gchar *description)
+{
+  AtkObject *aobj;
+  aobj = gtk_widget_get_accessible (widget);
+  /* return if gail is not loaded */
+  if (GTK_IS_ACCESSIBLE (aobj) == FALSE)
+     return;
+  atk_object_set_name (aobj, name);
+  atk_object_set_description (aobj, description);
+}
 
+static void
+make_applet_accessible (GtkWidget *applet)
+{
+  GtkTooltips *tooltips;
+  tooltips = gtk_tooltips_new ();
+  g_object_ref (tooltips);
+  gtk_object_sink (GTK_OBJECT (tooltips));
+  g_object_set_data (G_OBJECT (applet), "tooltips", tooltips); 
+  gtk_tooltips_set_tip (tooltips, applet, 
+                      _("Insert special characters"), NULL);
+  set_atk_name_description (applet, _("Character Palette"), NULL);
+}
 
 static gboolean
 charpicker_applet_fill (PanelApplet *applet)
 {
   charpick_data *curr_data;
   GtkWidget *event_box = NULL;
+  gchar *default_charlist_utf8;
   
   panel_applet_add_preferences (applet, "/schemas/apps/charpick/prefs", NULL);
    
@@ -416,10 +460,16 @@ charpicker_applet_fill (PanelApplet *applet)
   curr_data->event_box = event_box;
   curr_data->applet = GTK_WIDGET (applet);
   
-  curr_data->default_charlist = panel_applet_gconf_get_string (applet, 
-  							       "default_list", NULL);
-  if (!curr_data->default_charlist)
+  default_charlist_utf8 = panel_applet_gconf_get_string (applet, 
+  							 "default_list", NULL);
+  if (!default_charlist_utf8)
     curr_data->default_charlist = g_strdup (def_list);
+  else {
+    curr_data->default_charlist = g_convert (default_charlist_utf8, -1, 
+    					     "ISO-8859-1", "UTF-8", 
+  					      NULL, NULL, NULL);
+    g_free (default_charlist_utf8);
+  }
     
   curr_data->charlist = curr_data->default_charlist;
 
@@ -428,6 +478,7 @@ charpicker_applet_fill (PanelApplet *applet)
   event_box = gtk_event_box_new ();
   curr_data->event_box = event_box; 
   GTK_WIDGET_SET_FLAGS (event_box, GTK_CAN_FOCUS);
+  gtk_widget_set_events (curr_data->event_box, GDK_FOCUS_CHANGE_MASK|GDK_KEY_PRESS_MASK);
 
   /* Create table */
   curr_data->frame = gtk_frame_new (NULL);
@@ -437,7 +488,7 @@ charpicker_applet_fill (PanelApplet *applet)
   gtk_frame_set_shadow_type (GTK_FRAME(curr_data->frame), GTK_SHADOW_IN);
 
   /* Event signals */
-  gtk_signal_connect (GTK_OBJECT (event_box), "key_press_event",
+  gtk_signal_connect (GTK_OBJECT (curr_data->applet), "key_press_event",
 		      (GtkSignalFunc) key_press_event, curr_data);
   /*
   gtk_signal_connect (GTK_OBJECT (applet), "focus_in_event",
@@ -447,13 +498,15 @@ charpicker_applet_fill (PanelApplet *applet)
 		      (GtkSignalFunc) focus_out_event, NULL);
   */
 
-  gtk_widget_set_events (curr_data->event_box, GDK_FOCUS_CHANGE_MASK|GDK_KEY_PRESS_MASK);
-
   /* selection handling for selected character */
   gtk_selection_add_target (curr_data->event_box, 
 			    GDK_SELECTION_PRIMARY,
                             GDK_SELECTION_TYPE_STRING,
 			    0);
+  gtk_selection_add_target (curr_data->event_box, 
+			    GDK_SELECTION_CLIPBOARD,
+                            GDK_SELECTION_TYPE_STRING,
+			    0);
   gtk_signal_connect (GTK_OBJECT (curr_data->event_box), "selection_get",
 		      GTK_SIGNAL_FUNC (charpick_selection_handler),
 		      curr_data);
@@ -463,6 +516,7 @@ charpicker_applet_fill (PanelApplet *applet)
  
 
   gtk_container_add (GTK_CONTAINER (applet), curr_data->frame);
+  make_applet_accessible (GTK_WIDGET (applet));
   
   /* session save signal */ 
   g_signal_connect (G_OBJECT (applet), "change_orient",
@@ -470,13 +524,18 @@ charpicker_applet_fill (PanelApplet *applet)
 
   g_signal_connect (G_OBJECT (applet), "change_size",
 		    G_CALLBACK (applet_change_pixel_size), curr_data);
+		    
+  g_signal_connect (G_OBJECT (applet), "destroy",
+  		    G_CALLBACK (applet_destroy), curr_data);
   
   gtk_widget_show_all (GTK_WIDGET (applet));
   
-  panel_applet_setup_menu (PANEL_APPLET (applet),
-			   charpick_applet_menu_xml,
-			   charpick_applet_menu_verbs,
-			   curr_data);
+  panel_applet_setup_menu_from_file (PANEL_APPLET (applet),
+                                     NULL,
+			             "GNOME_CharpickerApplet.xml",
+                                     NULL,
+			             charpick_applet_menu_verbs,
+			             curr_data);
 
   return TRUE;
 }
@@ -495,6 +554,7 @@ charpicker_applet_factory (PanelApplet *applet,
 }
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_CharpickerApplet_Factory",
+			     PANEL_TYPE_APPLET,
 			     "Inserts characters",
 			     "0",
 			     charpicker_applet_factory,
