@@ -254,26 +254,32 @@ preadd_cb (GtkTreeSelection *selection,
   gtk_tree_model_get (GTK_TREE_MODEL(pbi->model), &iter,
                               NAME_COL, &value, -1);
   tdata->name = g_strdup (value);
+  g_free (value);
 
   gtk_tree_model_get (GTK_TREE_MODEL(pbi->model), &iter,
                               FLAG_COL, &value, -1);
   tdata->flag = g_strdup (value);
+  g_free (value);
 
   gtk_tree_model_get (GTK_TREE_MODEL(pbi->model), &iter,
                               COMMAND_COL, &value, -1);
   tdata->command = g_strdup (value);
+  g_free (value);
 
   gtk_tree_model_get (GTK_TREE_MODEL(pbi->model), &iter,
                               COUNTRY_COL, &value, -1);
   tdata->country = g_strdup (value);
+  g_free (value);
 
   gtk_tree_model_get (GTK_TREE_MODEL(pbi->model), &iter,
                               LABEL_COL, &value, -1);
   tdata->label = g_strdup (value);
+  g_free (value);
 
   gtk_tree_model_get (GTK_TREE_MODEL(pbi->model), &iter,
                               LANG_COL, &value, -1); 
   tdata->lang = g_strdup (value);
+  g_free (value);
 
   pbi->keymap_for_add = tdata;
 
@@ -308,13 +314,17 @@ addwadd_cb (GtkWidget * addbutton, GkbPropertyBoxInfo * pbi)
   return FALSE;
 }
 
-static void
-row_activated_cb (GtkTreeView *tree, GtkTreePath *path, 
-		  GtkTreeViewColumn *column, gpointer data)
+static gboolean
+row_activated_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
   GkbPropertyBoxInfo * pbi = data;
   
-  addwadd_cb (NULL, pbi);
+  if (event->type == GDK_2BUTTON_PRESS) {
+  	addwadd_cb (NULL, pbi);
+  	return TRUE;
+  }
+  
+  return FALSE;
   
 }
 
@@ -332,8 +342,7 @@ response_cb (GtkDialog *dialog, gint id, gpointer data)
     addhelp_cb (NULL, NULL);
     break;
   default:
-    gtk_widget_destroy (GTK_WIDGET (dialog));
-    gkb->addwindow = NULL;
+    gtk_widget_hide (GTK_WIDGET (dialog));
     break;
   }
  
@@ -350,6 +359,7 @@ gkb_prop_map_add (GkbPropertyBoxInfo * pbi)
  
   if (gkb->addwindow)
     {
+      gtk_widget_show (gkb->addwindow);
       gtk_window_present (GTK_WINDOW (gkb->addwindow));
       return;
     }
@@ -381,10 +391,8 @@ gkb_prop_map_add (GkbPropertyBoxInfo * pbi)
 
   gtk_widget_show (tree1);
 
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled1),
-					 tree1);
+  gtk_container_add (GTK_CONTAINER (scrolled1), tree1);
 
-  
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree1));
 
   gtk_tree_selection_set_mode (GTK_TREE_SELECTION (selection),
@@ -393,7 +401,7 @@ gkb_prop_map_add (GkbPropertyBoxInfo * pbi)
   g_signal_connect (selection, "changed",
 		      G_CALLBACK (preadd_cb), pbi);
   /* Signal for double clicks or user pressing space */
-  g_signal_connect (G_OBJECT (tree1), "row_activated",
+  g_signal_connect (G_OBJECT (tree1), "button_press_event",
                     G_CALLBACK (row_activated_cb), pbi);
        
   g_signal_connect (G_OBJECT (gkb->addwindow), "response",
