@@ -1,4 +1,4 @@
-/* $Id: gweather-about.c,v 1.5 2002/01/22 23:15:28 kevinv Exp $ */
+/* $Id: gweather-about.c,v 1.6 2002/01/30 03:25:30 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -41,7 +41,8 @@ void gweather_about_run (void)
     }
     about_dialog = gnome_about_new (_("GNOME Weather"), VERSION,
                                     _("Copyright (c)1999 by S. Papadimitriou"),
-                                    _("GNOME weather monitor applet.\nWeb: http://gweather.dhs.org/"),
+                                    _("Released under the GNU General Public License.\n\n"
+                                    	"An applet for monitoring local weather conditions."),
                                     authors,
                                     NULL,
                                     NULL,
