@@ -1,4 +1,4 @@
-/* $Id: gweather-dialog.c,v 1.14 2002/02/17 15:54:45 kmaraas Exp $ */
+/* $Id: gweather-dialog.c,v 1.18 2002/03/26 01:44:04 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -265,10 +265,10 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
   gtk_box_pack_end (GTK_BOX (cond_hbox), cond_frame_alignment, FALSE, FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (cond_frame_alignment), 2);
 
-  weather_info_get_pixmap (NULL, &(gw_applet->dialog_pixmap), &(gw_applet->dialog_mask));
-  gw_applet->cond_pixmap = gtk_pixmap_new (gw_applet->dialog_pixmap, gw_applet->dialog_mask);
-  gtk_widget_show (gw_applet->cond_pixmap);
-  gtk_container_add (GTK_CONTAINER (cond_frame_alignment), gw_applet->cond_pixmap);
+  weather_info_get_pixbuf (NULL, &(gw_applet->dialog_pixbuf));
+  gw_applet->cond_image = gtk_image_new_from_pixbuf (gw_applet->dialog_pixbuf);
+  gtk_widget_show (gw_applet->cond_image);
+  gtk_container_add (GTK_CONTAINER (cond_frame_alignment), gw_applet->cond_image);
 
   cond_vsep = gtk_vseparator_new ();
   gtk_widget_show (cond_vsep);
@@ -286,8 +286,11 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
 				  GTK_POLICY_AUTOMATIC,
 				  GTK_POLICY_AUTOMATIC);
 
-  gw_applet->forecast_text = gtk_text_new (NULL, NULL);
+  gw_applet->forecast_text = gtk_text_view_new ();
+  set_access_namedesc (gw_applet->forecast_text, _("ForeCast Report"),                                                             _("See the ForeCast Details"));
   gtk_container_add (GTK_CONTAINER (scrolled_window), gw_applet->forecast_text);
+  gtk_text_view_set_editable (GTK_TEXT_VIEW (gw_applet->forecast_text), FALSE);
+  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (gw_applet->forecast_text), GNOME_PAD);
   gtk_widget_show (gw_applet->forecast_text);
   gtk_widget_show (scrolled_window);
   gtk_box_pack_start (GTK_BOX (forecast_hbox), scrolled_window, TRUE, TRUE, 0);
@@ -299,7 +302,7 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (weather_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (weather_notebook), 1), forecast_note_lbl);
 
   if (gw_applet->gweather_pref.radar_enabled) {
-g_print ("draw radar \n");
+
       radar_vbox = gtk_vbox_new (FALSE, 6);
       gtk_widget_show (radar_vbox);
       gtk_container_add (GTK_CONTAINER (weather_notebook), radar_vbox);
@@ -314,9 +317,9 @@ g_print ("draw radar \n");
       gtk_box_pack_start (GTK_BOX (radar_vbox), ebox, FALSE, FALSE, 0);
 
 
-      gw_applet->radar_pixmap = gtk_pixmap_new (gw_applet->dialog_pixmap, gw_applet->dialog_mask);  /* Tmp hack */
-      gtk_widget_show (gw_applet->radar_pixmap);
-      gtk_container_add (GTK_CONTAINER (ebox), gw_applet->radar_pixmap);
+      gw_applet->radar_image = gtk_image_new_from_pixbuf (gw_applet->dialog_pixbuf);  /* Tmp hack */
+      gtk_widget_show (gw_applet->radar_image);
+      gtk_container_add (GTK_CONTAINER (ebox), gw_applet->radar_image);
 
       /* POP */
       gtk_widget_pop_colormap ();
@@ -327,6 +330,7 @@ g_print ("draw radar \n");
       gtk_box_pack_start (GTK_BOX (radar_vbox), radar_link_alignment, FALSE, FALSE, 0);
 
       radar_link_btn = gtk_button_new_with_label (_("Visit Weather.com"));
+      set_access_namedesc (radar_link_btn, _("URL link Button"),                                                            _("Click to Enter Weather.com"));
       gtk_widget_set_usize(radar_link_btn, 450, -2);
       gtk_widget_show (radar_link_btn);
       gtk_container_add (GTK_CONTAINER (radar_link_alignment), radar_link_btn);
@@ -371,7 +375,8 @@ void gweather_dialog_display_toggle (GWeatherApplet *gw_applet)
 void gweather_dialog_update (GWeatherApplet *gw_applet)
 {
     const gchar *forecast;
-    GdkFont* detailed_forecast_font = gdk_fontset_load ( "fixed" );
+    GtkTextBuffer *buffer;
+    /*GdkFont* detailed_forecast_font = gdk_fontset_load ( "fixed" );*/
 
     /* Check for parallel network update in progress */
     if(gw_applet->gweather_info == NULL)
@@ -380,9 +385,11 @@ void gweather_dialog_update (GWeatherApplet *gw_applet)
     if (!gw_applet->gweather_dialog)
         return;
 
-    /* Update pixmap */
-    weather_info_get_pixmap(gw_applet->gweather_info, &(gw_applet->dialog_pixmap), &(gw_applet->dialog_mask));
-    gtk_pixmap_set(GTK_PIXMAP(gw_applet->cond_pixmap), gw_applet->dialog_pixmap, gw_applet->dialog_mask);
+    /* Update pixbuf */
+    weather_info_get_pixbuf(gw_applet->gweather_info, 
+                            &(gw_applet->dialog_pixbuf));
+    gtk_image_set_from_pixbuf (GTK_IMAGE (gw_applet->cond_image), 
+                               gw_applet->dialog_pixbuf);
 
     /* Update current condition fields and forecast */
     gtk_label_set_text(GTK_LABEL(gw_applet->cond_location), weather_info_get_location(gw_applet->gweather_info));
@@ -397,23 +404,23 @@ void gweather_dialog_update (GWeatherApplet *gw_applet)
     gtk_label_set_text(GTK_LABEL(gw_applet->cond_vis), weather_info_get_visibility(gw_applet->gweather_info));
 
     /* Update forecast */
-    gtk_text_set_point(GTK_TEXT(gw_applet->forecast_text), 0);
-    gtk_text_forward_delete(GTK_TEXT(gw_applet->forecast_text), gtk_text_get_length(GTK_TEXT(gw_applet->forecast_text)));
+    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (gw_applet->forecast_text));
     forecast = weather_info_get_forecast(gw_applet->gweather_info);
     if (forecast) {
-        gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), detailed_forecast_font, NULL, NULL, forecast, strlen(forecast));
+        gtk_text_buffer_set_text(buffer, forecast, -1);
     } else {
         if (gw_applet->gweather_pref.detailed)
-            gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), detailed_forecast_font, NULL, NULL, _("Detailed forecast not available for this location.\nPlease try the state forecast; note that IWIN forecasts are available only for US cities."), -1);
+            gtk_text_buffer_set_text(buffer, _("Detailed forecast not available for this location.\nPlease try the state forecast; note that IWIN forecasts are available only for US cities."), -1);
         else
-            gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), detailed_forecast_font, NULL, NULL, _("State forecast not available for this location.\nPlease try the detailed forecast; note that IWIN forecasts are available only for US cities."), -1);
+            gtk_text_buffer_set_text(buffer, _("State forecast not available for this location.\nPlease try the detailed forecast; note that IWIN forecasts are available only for US cities."), -1);
     }
 
     /* Update radar map */
     if (gw_applet->gweather_pref.radar_enabled) {
         GdkPixmap *radar = weather_info_get_radar(gw_applet->gweather_info);
         if (radar) {
-            gtk_pixmap_set (GTK_PIXMAP(gw_applet->radar_pixmap), radar, NULL);
+            gtk_image_set_from_pixmap (GTK_IMAGE (gw_applet->radar_image), 
+                                       radar, NULL);
         }
     }
 }
