@@ -53,6 +53,13 @@ applet_destroy_signal(GtkWidget *widget, gpointer data)
 {
     MCData *mcdata = data;
     PanelApplet *applet = mcdata->applet;
+    GtkTooltips *tooltips;
+	
+    tooltips = g_object_get_data (G_OBJECT (applet), "tooltips");
+    if (tooltips) {
+        g_object_unref (tooltips);
+        g_object_set_data (G_OBJECT (applet), "tooltips", NULL);
+    }
     
     /* applet will be destroyed; save history */
     save_session();
@@ -154,6 +161,20 @@ applet_pixel_size_changed_cb(GtkWidget *widget, int size, gpointer data)
     data = NULL;
 }
 
+void
+set_atk_name_description(GtkWidget *widget, const gchar *name,
+const gchar *description)
+{	
+    AtkObject *aobj;
+	
+    aobj = gtk_widget_get_accessible(widget);
+    if (GTK_IS_ACCESSIBLE (aobj) == FALSE)
+        return;
+
+    atk_object_set_name(aobj, name);
+    atk_object_set_description(aobj, description);
+}
+
 static gboolean
 button_press_hack (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
@@ -181,11 +202,15 @@ redraw_applet(MCData *mcdata)
     GtkWidget *icon;
     GtkWidget *vbox;
     GtkTooltips *tooltips;
+    GdkPixbuf *image;
     int size_frames = 0;
     int size_status_line = 18;
     gboolean first_time = FALSE;
 
     tooltips = gtk_tooltips_new ();
+    g_object_ref (tooltips);
+    gtk_object_sink (GTK_OBJECT (tooltips));
+    g_object_set_data (G_OBJECT (applet), "tooltips", tooltips); 
     
     /* recalculate sizes */
     if(prop->show_handle)
@@ -257,11 +282,15 @@ redraw_applet(MCData *mcdata)
     g_signal_connect (G_OBJECT (button), "button_press_event",
 		      G_CALLBACK (button_press_hack), applet);
     gtk_widget_set_usize(GTK_WIDGET(button), 13, 10);
-    icon = gnome_pixmap_new_from_xpm_d (browser_mini_xpm);
+    image = gdk_pixbuf_new_from_xpm_data (browser_mini_xpm);
+    icon = gtk_image_new_from_pixbuf (image);
     gtk_container_add(GTK_CONTAINER(button), icon);
 
     gtk_tooltips_set_tip (tooltips, button, _("Browser"), NULL);
     gtk_box_pack_start(GTK_BOX(hbox_buttons), button, TRUE, TRUE, 0);
+	
+    set_atk_name_description(button, _("Browser"),
+        _("Click this button to start the browser"));
 
     /* add history button */
     button = gtk_button_new();
@@ -271,12 +300,18 @@ redraw_applet(MCData *mcdata)
     g_signal_connect (G_OBJECT (button), "button_press_event",
 		      G_CALLBACK (button_press_hack), applet);
     gtk_widget_set_usize(GTK_WIDGET(button), 13, 10);
-    icon = gnome_pixmap_new_from_xpm_d (history_mini_xpm);
+    image = gdk_pixbuf_new_from_xpm_data (history_mini_xpm);
+    icon = gtk_image_new_from_pixbuf (image);
     gtk_container_add(GTK_CONTAINER(button), icon);
 
     gtk_tooltips_set_tip (tooltips, button, _("History"), NULL);
     gtk_box_pack_end(GTK_BOX(hbox_buttons), button, TRUE, TRUE, 0);
 
+    set_atk_name_description(button, _("History"),
+        _("Click this button for the list of previous commands"));
+    set_atk_name_description(GTK_WIDGET(applet), _("Mini-Commander applet"),
+        _("This applet adds a command line to the panel"));
+	
     /* add buttons into frame */
     frame = gtk_frame_new(NULL);
     gtk_container_set_border_width(GTK_CONTAINER(frame), 1);
@@ -319,8 +354,7 @@ redraw_applet(MCData *mcdata)
 		    gtk_container_add(GTK_CONTAINER(frame2), handle);
 		}
 	    
-	    /* there was trouble with the tooltip */
-	    /* applet_widget_set_tooltip(APPLET_WIDGET(applet),  _("Mini-Commander")); */
+	    gtk_tooltips_set_tip (tooltips, GTK_WIDGET (applet),  _("Command Line"), NULL);
 	    
 	    if (prop->show_frame)
 		gtk_box_pack_start(GTK_BOX(mcdata->applet_inner_vbox), frame2, TRUE, TRUE, 0);
