@@ -24,6 +24,7 @@
 #include <panel-applet.h>
 #include <gconf/gconf.h>
 #include <string.h>
+#include <panel-applet-gconf.h>
 
 #include "preferences.h"
 #include "command_line.h" /* needed for style changes */
@@ -100,6 +101,33 @@ color_cmd_bg_changed_signal(GtkWidget *color_picker_widget,
     data = NULL;
 }
 
+static void
+set_atk_relation (GtkWidget *label, GtkWidget *widget)
+{
+    AtkObject *atk_widget;
+    AtkObject *atk_label;
+    AtkRelationSet *relation_set;
+    AtkRelation *relation;
+    AtkObject *targets[1];
+
+    atk_widget = gtk_widget_get_accessible (widget);
+    atk_label = gtk_widget_get_accessible (label);
+
+    /* Set label-for relation */
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);	
+
+    /* Check if gail is loaded */
+    if (GTK_IS_ACCESSIBLE (atk_widget) == FALSE)
+        return;
+    
+    /* Set labelled-by relation */
+    relation_set = atk_object_ref_relation_set (atk_widget);
+    targets[0] = atk_label;
+    relation = atk_relation_new (targets, 1, ATK_RELATION_LABELLED_BY);
+    atk_relation_set_add (relation_set, relation);
+    g_object_unref (G_OBJECT (relation)); 
+}
+
 static void
 properties_box_apply_signal(GnomePropertyBox *property_box_widget, gint page, gpointer data)
 {
@@ -494,6 +522,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 						  GTK_DIALOG_DESTROY_WITH_PARENT,
 						  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 						  NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG (mcdata->properties_box), GTK_RESPONSE_CLOSE);
     gtk_window_set_default_size (GTK_WINDOW (mcdata->properties_box), 400, 300);
     
     notebook = gtk_notebook_new ();
@@ -535,14 +564,14 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_container_add(GTK_CONTAINER(frame), vbox1);
 
     /* show handle check box */
-    check_handle = gtk_check_button_new_with_label (_("Show handle"));
+    check_handle = gtk_check_button_new_with_mnemonic (_("Show han_dle"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_handle), prop->show_handle);
     g_signal_connect (G_OBJECT (check_handle), "toggled",
     		      G_CALLBACK (check_handle_toggled), mcdata);
     gtk_box_pack_start(GTK_BOX(vbox1), check_handle, FALSE, TRUE, 0);
 
     /* show frame check box */
-    check_frame = gtk_check_button_new_with_label (_("Show frame"));
+    check_frame = gtk_check_button_new_with_mnemonic (_("Show fram_e"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_frame), prop->show_frame);
     g_signal_connect (G_OBJECT (check_frame), "toggled",
     		      G_CALLBACK (check_frame_toggled), mcdata);
@@ -559,7 +588,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_container_add(GTK_CONTAINER(frame), vbox1);
 
     /* show history autocomplete */
-    check_auto_complete_history = gtk_check_button_new_with_label (_("Enable history based autocompletion"));
+    check_auto_complete_history = gtk_check_button_new_with_mnemonic (_("Enable _history based autocompletion"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_auto_complete_history), prop->auto_complete_history);
     g_signal_connect (G_OBJECT (check_auto_complete_history), "toggled",
     		      G_CALLBACK (autocomplete_toggled), mcdata);
@@ -578,7 +607,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_container_add(GTK_CONTAINER(frame), table);
 
     /* applet width */    
-    label = gtk_label_new(_("Applet width:"));
+    label = gtk_label_new_with_mnemonic(_("Applet _width:"));
     gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), 
 		     label,
@@ -594,6 +623,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 		     0, 1,
 		     GTK_FILL, 0,
 		     0, 0);
+    set_atk_relation (label, spin),
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), prop->normal_size_x);
     g_signal_connect (G_OBJECT (spin), "value_changed",
     		      G_CALLBACK (size_changed_cb), mcdata); 
@@ -621,6 +651,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 		     0, 0);
 
     entry = gtk_entry_new_with_max_length(4);
+    set_atk_relation (label, entry);
     g_snprintf(buffer, sizeof(buffer), "%d", prop->normal_size_y);
     gtk_entry_set_text(GTK_ENTRY(entry), (gchar *) buffer);
     gtk_widget_set_usize(entry, 50, -1);
@@ -642,6 +673,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 		     0, 0);
 
     entry = gtk_entry_new_with_max_length(4);
+    set_atk_relation (label, entry);
     g_snprintf(buffer, sizeof(buffer), "%d", prop->cmd_line_size_y);
     gtk_entry_set_text(GTK_ENTRY(entry), (gchar *) buffer);
     gtk_widget_set_usize(entry, 50, -1);
@@ -676,7 +708,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_container_add(GTK_CONTAINER(frame), table);
 
     /* fg */
-    label = gtk_label_new(_("Command line foreground:"));
+    label = gtk_label_new_with_mnemonic(_("Command line _foreground:"));
     gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), 
 		     label,
@@ -691,6 +723,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 			       prop->cmd_line_color_fg_g, 
 			       prop->cmd_line_color_fg_b, 
 			       0);
+    set_atk_relation(label, color_picker);
     gtk_signal_connect(GTK_OBJECT(color_picker),
 		       "color_set",
 		       GTK_SIGNAL_FUNC(color_cmd_fg_changed_signal),
@@ -707,7 +740,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     
 
     /* bg */
-    label = gtk_label_new(_("Command line background:"));
+    label = gtk_label_new_with_mnemonic(_("Command line _background:"));
     gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), 
 		     label,
@@ -721,6 +754,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 			       prop->cmd_line_color_bg_g, 
 			       prop->cmd_line_color_bg_b, 
 			       0);
+    set_atk_relation(label, color_picker);
     gtk_signal_connect(GTK_OBJECT(color_picker),
 		       "color_set",
 		       GTK_SIGNAL_FUNC(color_cmd_bg_changed_signal),
@@ -761,12 +795,14 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 	    
 	    /* prefix */    
 	    g_snprintf(text_label, sizeof(text_label), _("Regex %.2d:"), i+1);
-	    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(text_label), FALSE, TRUE, 0);
+	    label = gtk_label_new(text_label);
+	    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
 	    
 	    entry = gtk_entry_new_with_max_length(MAX_MACRO_PATTERN_LENGTH);
 	    gtk_widget_set_usize(entry, 75, -1);
 	    if (prop->macro_pattern[i] != (gchar *) NULL)
 		gtk_entry_set_text(GTK_ENTRY(entry), (gchar *) prop->macro_pattern[i]);
+	    set_atk_relation(label, entry);
 	    #ifdef FIXME
 	    gtk_signal_connect(GTK_OBJECT(entry),
 			       "changed",
@@ -777,11 +813,13 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 	    
 	    /* command */
 	    g_snprintf(text_label, sizeof(text_label), _("   Macro %.2d:"), i+1);
-	    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(text_label), FALSE, TRUE, 0);
+	    label = gtk_label_new(text_label);
+	    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
 	    
 	    entry = gtk_entry_new_with_max_length(MAX_COMMAND_LENGTH);
 	    if (prop->macro_command[i] != (gchar *) NULL)
 		gtk_entry_set_text(GTK_ENTRY(entry), prop->macro_command[i]);
+	    set_atk_relation(label, entry);
 	    #ifdef FIXME
 	    gtk_signal_connect(GTK_OBJECT(entry),
 			       "changed",
