@@ -20,6 +20,7 @@
 #endif
 
 #include <libbonobo.h>
+#include <bonobo/bonobo-file-selector-util.h>
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
 #include <panel-applet.h>
@@ -72,6 +73,10 @@ typedef struct _PanelMenuPath {
 static void panel_menu_path_set_list (PanelMenuEntry *entry, GList *list);
 static void regenerate_menus_cb (GtkWidget *menuitem, PanelMenuEntry *entry,
 				 const gchar *verb);
+static void monitor_path_cb(GnomeVFSMonitorHandle *handle, const gchar *monitor_uri,
+			    const gchar *info_uri, GnomeVFSMonitorEventType event_type,
+			    gpointer user_data);
+static void kill_monitor_cb (GtkWidget *widget, gpointer data);
 static void directory_load_cb (GnomeVFSAsyncHandle *handle,
 			       GnomeVFSResult result, GList *list,
 			       guint entries_read, GtkMenuShell *parent);
@@ -80,11 +85,6 @@ static void panel_menu_path_process (GnomeVFSFileInfo *finfo,
 				     GtkMenuShell *parent,
 				     const gchar *subpath);
 
-static void monitor_path_cb(GnomeVFSMonitorHandle *handle, const gchar *monitor_uri,
-			    const gchar *info_uri, GnomeVFSMonitorEventType event_type,
-			    gpointer user_data);
-static void kill_monitor_cb (GtkWidget *widget, gpointer data);
-
 static gint panel_menu_path_remove_cb (GtkWidget *widget, GdkEventKey *event,
 				       PanelMenuPath *path);
 static void change_path_cb (GtkWidget *widget, PanelMenuEntry *entry, const char *verb);
@@ -156,6 +156,7 @@ panel_menu_path_new_with_id (PanelMenu *parent, gint id)
 	client = gconf_client_get_default ();
 	if (gconf_client_dir_exists (client, dir_key, NULL)) {
 		gchar *key;
+
 		key = g_strdup_printf ("%s/base-uri", base_key);
 		uri = panel_applet_gconf_get_string (parent->applet,
 						     key, NULL);
@@ -165,10 +166,12 @@ panel_menu_path_new_with_id (PanelMenu *parent, gint id)
 		panel_menu_path_set_list (entry,
 			panel_applet_gconf_get_string_list (parent->applet, key));
 		g_free (key);
+		
 	} else {
 		uri = g_strdup ("applications:");
 		panel_menu_path_set_uri (entry, uri);
 	}
+
 	g_object_unref (G_OBJECT (client));
 	g_free (base_key);
 	g_free (dir_key);
@@ -233,7 +236,9 @@ panel_menu_path_set_uri (PanelMenuEntry *entry, gchar *uri)
 		label = g_strdup (_("Programs"));
 	gtk_label_set_text (GTK_LABEL (GTK_BIN (path->path)->child), label);
 	g_free (label);
+	kill_monitor_cb (path->path, NULL);
 	regenerate_menus_cb (NULL, entry, NULL);
+	panel_menu_path_monitor (path->base_path, GTK_MENU_ITEM (path->path));
 }
 
 static void
@@ -355,6 +360,91 @@ panel_menu_path_accept_drop (PanelMenuEntry *entry, GnomeVFSURI *uri)
 	return retval;
 }
 
+void
+panel_menu_path_monitor (const gchar *uri, GtkMenuItem *menu_item)
+{
+	GnomeVFSResult monitor_result;
+	GnomeVFSMonitorHandle *monitor = NULL;
+
+	if ((monitor = g_object_get_data (G_OBJECT (menu_item), "vfs-monitor"))) {
+		kill_monitor_cb (GTK_WIDGET (menu_item), monitor);
+		monitor = NULL;
+	}
+	monitor_result = gnome_vfs_monitor_add (&monitor, uri,
+						GNOME_VFS_MONITOR_DIRECTORY,
+						monitor_path_cb, menu_item);
+	if (monitor_result == GNOME_VFS_OK) {
+		g_object_set_data (G_OBJECT (menu_item), "vfs-monitor", monitor);
+		g_signal_connect (G_OBJECT (menu_item), "destroy",
+				  G_CALLBACK (kill_monitor_cb), NULL);
+	} else {
+		g_print ("monitor installation failed for path %s\n", uri);
+	}
+}
+
+static void
+monitor_path_cb(GnomeVFSMonitorHandle *handle,
+		const gchar *monitor_uri,
+		const gchar *info_uri,
+		GnomeVFSMonitorEventType event_type,
+		gpointer user_data)
+{
+	GtkMenuItem *menuitem;
+	menuitem = GTK_MENU_ITEM (user_data);
+
+	switch (event_type) {
+		case GNOME_VFS_MONITOR_EVENT_CHANGED:
+			//g_print ("GNOME_VFS_MONITOR_EVENT_CHANGED");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_DELETED:
+			//g_print ("GNOME_VFS_MONITOR_EVENT_DELETED");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
+			//g_print ("GNOME_VFS_MONITOR_EVENT_STARTEXECUTING");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
+			//g_print ("GNOME_VFS_MONITOR_EVENT_STOPEXECUTING");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_CREATED:
+			//g_print ("GNOME_VFS_MONITOR_EVENT_CREATED");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
+			//g_print ("GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED");
+			break;
+		default:
+			break;
+	}
+	if (menuitem->submenu) {
+		GtkMenuShell *menu;
+		GList *list;
+		GList *cur;
+		gchar *path;
+
+		menu = GTK_MENU_SHELL (menuitem->submenu);
+		list = g_list_copy (menu->children);
+		for (cur = list; cur; cur = cur->next) {
+			if (g_object_get_data (G_OBJECT (cur->data), "uri-path"))
+				gtk_widget_destroy (GTK_WIDGET (cur->data));
+		}
+		g_list_free (list);
+		g_print ("Path %s has changed, regenerating menu...\n", monitor_uri);
+		panel_menu_path_load (monitor_uri, menu);
+	}
+}
+
+static void
+kill_monitor_cb (GtkWidget *widget, gpointer data)
+{
+	GnomeVFSMonitorHandle *monitor;
+	
+	monitor = (GnomeVFSMonitorHandle *) g_object_get_data (G_OBJECT (widget),
+							      "vfs-monitor");
+	if (monitor) {
+		gnome_vfs_monitor_cancel (monitor);
+		g_object_set_data (G_OBJECT (widget), "vfs-monitor", NULL);
+	}
+}
+
 void
 panel_menu_path_load (const gchar *uri, GtkMenuShell *parent)
 {
@@ -383,7 +473,6 @@ directory_load_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	gchar *path;
 	gchar *subpath = NULL;
 	gint count = 0;
-	GnomeVFSResult monitor_result;
 
 	path = (gchar *) g_object_get_data (G_OBJECT (parent), "uri-path");
 	for (iter = list, count = 0; iter && count < entries_read;
@@ -400,14 +489,10 @@ directory_load_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	if (result != GNOME_VFS_OK) {
 		if (parent) {
 			GtkWidget *w = NULL;
-			GnomeVFSMonitorHandle *monitor = NULL;
 
 			subpath =
 				(gchar *) g_object_get_data (G_OBJECT (parent),
 							     "uri-path");
-			if (subpath)
-			  printf ("Subpath is %s\n", subpath);
-			else printf ("Subpath WAS NULL\n");
 			g_object_set_data (G_OBJECT (parent), "uri-path", NULL);
 			w = GTK_MENU (parent)->parent_menu_item;
 			if (w && g_list_length (parent->children) < 2 &&
@@ -419,24 +504,6 @@ directory_load_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 				panel_menu_pixbuf_set_icon (GTK_MENU_ITEM (w),
 							   "directory");
 			}
-			if (w && subpath) {
-			        monitor_result = gnome_vfs_monitor_add (&monitor,
-								subpath,
-								GNOME_VFS_MONITOR_DIRECTORY,
-								monitor_path_cb, w);
-
-				if (monitor_result == GNOME_VFS_OK) {
-				  /* g_print ("monitor successfully installed for %s\n",
-				     subpath); */
-					g_object_set_data (G_OBJECT (w), "vfs-monitor", monitor);
-					g_signal_connect (G_OBJECT (w), "destroy",
-							  G_CALLBACK (kill_monitor_cb),
-							  monitor);
-				} else {
-					g_print ("monitor installation failed for path %s\n",
-						  subpath);
-				}
-			}
 			g_free (subpath);
 		}
 	}
@@ -478,71 +545,6 @@ panel_menu_path_process (GnomeVFSFileInfo *finfo, GtkMenuShell *parent,
 	}
 }
 
-static void
-monitor_path_cb(GnomeVFSMonitorHandle *handle,
-		const gchar *monitor_uri,
-		const gchar *info_uri,
-		GnomeVFSMonitorEventType event_type,
-		gpointer user_data)
-{
-	GtkMenuItem *menuitem;
-	menuitem = GTK_MENU_ITEM (user_data);
-
-	switch (event_type) {
-		case GNOME_VFS_MONITOR_EVENT_CHANGED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_CHANGED");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_DELETED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_DELETED");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
-			g_print ("GNOME_VFS_MONITOR_EVENT_STARTEXECUTING");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
-			g_print ("GNOME_VFS_MONITOR_EVENT_STOPEXECUTING");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_CREATED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_CREATED");
-			break;
-		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
-			g_print ("GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED");
-			break;
-		default:
-			break;
-	}
-	g_print (" (%s)", info_uri);
-	g_print ("\n");
-	if (menuitem->submenu) {
-		GtkMenuShell *menu;
-		GList *list;
-		GList *cur;
-		gchar *path;
-
-		menu = GTK_MENU_SHELL (menuitem->submenu);
-		list = g_list_copy (menu->children);
-		for (cur = list; cur; cur = cur->next) {
-			if (g_object_get_data (G_OBJECT (cur->data), "uri-path"))
-				gtk_widget_destroy (GTK_WIDGET (cur->data));
-		}
-		g_list_free (list);
-		path = g_object_get_data (G_OBJECT (menuitem), "uri-path");
-		if (path) {
-			panel_menu_path_load (path, menu);
-			g_print ("Path %s has changed, regenerating menu...\n", path);
-		}
-	}
-}
-
-static void
-kill_monitor_cb (GtkWidget *widget, gpointer data)
-{
-	GnomeVFSMonitorHandle *monitor;
-	
-	monitor = (GnomeVFSMonitorHandle *)data;
-	if (monitor)
-		gnome_vfs_monitor_cancel (monitor);
-}
-
 gboolean
 panel_menu_path_append_item (PanelMenuEntry *entry, gchar *uri)
 {
@@ -587,6 +589,8 @@ panel_menu_path_append_item (PanelMenuEntry *entry, gchar *uri)
 			menuitem = GTK_MENU (submenu)->parent_menu_item;
 			panel_menu_path_load ((const gchar *) uri,
 					      GTK_MENU_SHELL (submenu));
+			panel_menu_path_monitor ((const gchar *) uri,
+						 GTK_MENU_ITEM (menuitem));
 			retval = TRUE;
 		}
 		if (retval) {
@@ -638,6 +642,28 @@ panel_menu_path_remove_cb (GtkWidget *widget, GdkEventKey *event,
 	return retval;
 }
 
+static void
+handle_new_path_response (GtkDialog *dialog, gint response, PanelMenu *panel_menu)
+{
+	if (response == GTK_RESPONSE_OK) {
+		GtkEntry *path_entry;
+		const gchar *path;
+		PanelMenuEntry *entry;
+
+		path_entry = GTK_ENTRY (g_object_get_data (G_OBJECT (dialog),
+							   "source"));
+		path = gtk_entry_get_text (GTK_ENTRY (path_entry));
+		entry = panel_menu_path_new (panel_menu, (gchar *) path);
+		panel_menu->entries =
+			g_list_append (panel_menu->entries, (gpointer) entry);
+		gtk_menu_shell_append (GTK_MENU_SHELL (panel_menu->menubar),
+				       panel_menu_path_get_widget (entry));
+		panel_menu_config_save_layout (panel_menu);
+		panel_menu_path_save_config (entry);
+	}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 void
 panel_menu_path_new_with_dialog (PanelMenu *panel_menu)
 {
@@ -649,21 +675,32 @@ panel_menu_path_new_with_dialog (PanelMenu *panel_menu)
 
 	dialog = panel_menu_path_edit_dialog_new (_("Create menu path item..."),
 						  "applications:", &path_entry);
+	g_object_set_data (G_OBJECT (dialog), "source", path_entry);
+	g_signal_connect (G_OBJECT (dialog), "response",
+			  G_CALLBACK (handle_new_path_response),
+			  panel_menu);
 	gtk_widget_show (dialog);
 	gtk_widget_grab_focus (path_entry);
+}
 
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
+static void
+handle_change_path_response (GtkDialog *dialog, gint response, PanelMenuEntry *entry)
+{
 	if (response == GTK_RESPONSE_OK) {
-		path = (gchar *) gtk_entry_get_text (GTK_ENTRY (path_entry));
-		entry = panel_menu_path_new (panel_menu, path);
-		panel_menu->entries =
-			g_list_append (panel_menu->entries, (gpointer) entry);
-		gtk_menu_shell_append (GTK_MENU_SHELL (panel_menu->menubar),
-				       panel_menu_path_get_widget (entry));
-		panel_menu_config_save_layout (panel_menu);
-		panel_menu_path_save_config (entry);
+		GtkEntry *path_entry;
+		const gchar *new_path;
+		PanelMenuPath *path;
+
+		path = (PanelMenuPath *) entry->data;
+		path_entry = GTK_ENTRY (g_object_get_data (G_OBJECT (dialog),
+							   "source"));
+		new_path = gtk_entry_get_text (path_entry);
+		if (strcmp (path->base_path, new_path)) {
+			panel_menu_path_set_uri (entry, (gchar *)new_path);
+			panel_menu_path_save_config (entry);
+		}
 	}
-	gtk_widget_destroy (dialog);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static void
@@ -673,7 +710,6 @@ change_path_cb (GtkWidget *widget, PanelMenuEntry *entry, const char *verb)
 	GtkWidget *dialog;
 	GtkWidget *path_entry;
 	gint response;
-	gchar *new_path;
 
 	g_return_if_fail (entry != NULL);
 	g_return_if_fail (entry->type == PANEL_MENU_TYPE_PATH);
@@ -681,24 +717,52 @@ change_path_cb (GtkWidget *widget, PanelMenuEntry *entry, const char *verb)
 	path = (PanelMenuPath *) entry->data;
 	dialog = panel_menu_path_edit_dialog_new (_("Edit path item..."),
 						  path->base_path, &path_entry);
+	g_object_set_data (G_OBJECT (dialog), "source", path_entry);
+	g_signal_connect (G_OBJECT (dialog), "response",
+			  G_CALLBACK (handle_change_path_response),
+			  entry);
 	gtk_widget_show (dialog);
 	gtk_widget_grab_focus (path_entry);
+}
+
+static void
+handle_browse_response (GtkDialog *dialog, gint response, GtkFileSelection *fsel)
+{
+	GtkEntry *entry;
+	const gchar *new_path;
 
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (response == GTK_RESPONSE_OK) {
-		new_path =
-			(gchar *) gtk_entry_get_text (GTK_ENTRY (path_entry));
-		if (strcmp (path->base_path, new_path)) {
-			panel_menu_path_set_uri (entry, new_path);
-			panel_menu_path_save_config (entry);
+		new_path = gtk_file_selection_get_filename (fsel);
+		if (new_path) {
+			entry = GTK_ENTRY(g_object_get_data (G_OBJECT (dialog), "target"));
+			gtk_entry_set_text (entry, new_path);
 		}
 	}
-	gtk_widget_destroy (dialog);
+	gtk_widget_destroy (GTK_WIDGET(dialog));
+}
+
+static void
+browse_callback (GtkButton *button, GtkEntry *entry)
+{
+	GtkWidget *fsel;
+	const gchar *old_path;
+
+	fsel = gtk_file_selection_new ("Choose a directory...");
+	gtk_window_set_transient_for (GTK_WINDOW (fsel), GTK_WINDOW (
+				      gtk_widget_get_toplevel (GTK_WIDGET (entry))));
+	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
+	old_path = gtk_entry_get_text (entry);
+	gtk_file_selection_set_filename (GTK_FILE_SELECTION (fsel), old_path);
+	gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (fsel), FALSE);
+	g_object_set_data (G_OBJECT (fsel), "target", entry);
+	g_signal_connect (G_OBJECT (fsel), "response",
+			  G_CALLBACK (handle_browse_response), fsel);
+	gtk_widget_show (fsel);
 }
 
 static GtkWidget *
 panel_menu_path_edit_dialog_new (gchar *title, gchar *value,
-				 GtkWidget ** entry)
+				 GtkWidget **entry)
 {
 	GtkWidget *dialog;
 	GtkWidget *box;
@@ -727,8 +791,12 @@ panel_menu_path_edit_dialog_new (gchar *title, gchar *value,
 	gtk_widget_show (*entry);
 	gtk_entry_set_text (GTK_ENTRY (*entry), value);
 
+	set_relation (*entry, GTK_LABEL(label));
+
 	browse = gtk_button_new_with_label (_("Browse..."));
 	gtk_box_pack_start (GTK_BOX (hbox), browse, FALSE, FALSE, 5);
+	g_signal_connect (G_OBJECT (browse), "clicked",
+			  G_CALLBACK(browse_callback), *entry);
 	gtk_widget_show (browse);
 	return dialog;
 }
@@ -756,6 +824,7 @@ panel_menu_path_save_config (PanelMenuEntry *entry)
 	key = g_strdup_printf ("%s/paths-list", id);
 	panel_applet_gconf_set_string_list (applet, key, path->paths_list);
 	g_free (key);
+
 	return id;
 }
 
