@@ -1,4 +1,4 @@
-/*  panel-menu.c
+/* panel-menu.c
  * (c) 2001 Chris Phelps
  * Menubar based Panel Applet
  *
@@ -17,6 +17,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libbonobo.h>
 #include <bonobo/bonobo-shlib-factory.h>
 #include <libgnome/libgnome.h>
@@ -130,13 +134,14 @@ static const char applet_menu_xml[] =
 	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
 	"</popup>\n";
 
-
 #if MENU_APPLET_IS_SHLIB
 PANEL_APPLET_BONOBO_SHLIB_FACTORY ("OAFIID:GNOME_PanelMenuApplet_Factory",
+				   PANEL_TYPE_APPLET,
 				   "PanelMenu-Applet-Factory",
 				   applet_factory, NULL);
 #else
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_PanelMenuApplet_Factory",
+			     PANEL_TYPE_APPLET,
                              "PanelMenu-Applet-Factory",
                              "0",
                              applet_factory,
@@ -164,7 +169,6 @@ panel_menu_construct_applet (PanelApplet *applet)
 	panel_menu = g_new0 (PanelMenu, 1);
 
 	panel_menu->applet = applet;
-	panel_applet_set_expand_flags (panel_menu->applet, FALSE, FALSE);
 	panel_menu->size = panel_applet_get_size (panel_menu->applet);
 
 	key = panel_applet_get_preferences_key (applet);
@@ -581,6 +585,8 @@ applet_destroy_cb (GtkWidget *widget, PanelMenu *panel_menu)
 	PanelMenuEntry *entry;
 	GList *cur;
 
+	destroy_tooltip( widget );
+
 	g_free (panel_menu->profile_id);
 	g_free (panel_menu->applet_id);
 
@@ -685,3 +691,70 @@ applet_about_cb (BonoboUIComponent *uic, PanelMenu *panel_menu,
 			  G_CALLBACK (gtk_widget_destroyed), &about);
 	gtk_widget_show (about);
 }
+
+void
+add_tooltip (GtkWidget *widget, gchar *description)
+{
+	GtkTooltips *tooltips;
+
+	tooltips = g_object_get_data (G_OBJECT (widget), "tooltips");
+
+	/* create if not already present */
+	if (!tooltips) {
+		tooltips = gtk_tooltips_new ();
+		g_return_if_fail (tooltips != NULL);
+		g_object_ref (tooltips);
+		gtk_object_sink (GTK_OBJECT (tooltips));
+		g_object_set_data (G_OBJECT (widget), "tooltips", tooltips);
+	}
+
+	gtk_tooltips_set_tip (tooltips, widget, description, NULL);
+}
+
+void
+destroy_tooltip (GtkWidget *object)
+{
+        GtkTooltips *tooltips;
+
+        tooltips = g_object_get_data (G_OBJECT (object), "tooltips");
+        if (tooltips) {
+                g_object_unref (tooltips);
+                g_object_set_data (G_OBJECT (object), "tooltips", NULL);
+        }
+}
+
+/**
+ * set_relation
+ * @widget : The Gtk widget which is labelled by @label
+ * @label : The label for the @widget.
+ * Description : This function establishes atk relation
+ * between a gtk widget and a label.
+ */
+void
+set_relation (GtkWidget *widget, GtkLabel *label)
+{
+	AtkObject *aobject;
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *targets[1];
+
+	g_return_if_fail (GTK_IS_WIDGET(widget));
+	g_return_if_fail (GTK_IS_LABEL(label));
+
+	aobject = gtk_widget_get_accessible (widget);
+
+	/* Return if GAIL is not loaded */
+	if (! GTK_IS_ACCESSIBLE (aobject))
+		return;
+
+	/* Set the ATK_RELATION_LABEL_FOR relation */
+	gtk_label_set_mnemonic_widget (label, widget);
+
+	targets[0] = gtk_widget_get_accessible (GTK_WIDGET (label));
+
+	relation_set = atk_object_ref_relation_set (aobject);
+
+	relation = atk_relation_new (targets, 1, ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+}
